@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 18:45:55 1995
  * Project:     INN (innfeed)
  * File:        article.c
- * RCSId:       $Id: article.c,v 1.16 1996/08/12 09:07:56 brister Exp $
+ * RCSId:       $Id: article.c,v 1.19 1997/01/13 00:28:44 brister Exp $
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
  *              Permission to use, copy, modify, and distribute this
@@ -37,7 +37,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: article.c,v 1.16 1996/08/12 09:07:56 brister Exp $" ;
+static const char *rcsid = "$Id: article.c,v 1.19 1997/01/13 00:28:44 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -52,10 +52,14 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include <stdlib.h>
 #include <assert.h>
 #include <sys/types.h>
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif /* HAVE_MMAP */
 #include <sys/stat.h>
 #include <syslog.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <string.h>
 #include <limits.h>
 
 #include "article.h"
@@ -69,6 +73,7 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #define VALIDATE_HASH_TABLE() hashValidateTable ()
 #endif
 
+extern bool useMMap ;
 
 struct article_s 
 {
@@ -77,8 +82,10 @@ struct article_s
     char *msgid ;               /* the msgid of the article (INN tells us) */
     Buffer contents ;           /* the buffer of the actual on disk stuff */
     Buffer *nntpBuffers ;       /* list of buffers for transmisson */
+    caddr_t mMapping ;          /* base of memory mapping, or NULL if none */
     bool loggedMissing ;        /* true if article is missing and we logged */
     bool articleOk ;            /* true until we know otherwise. */
+    bool inWireFormat ;         /* true if ->contents is \r\n/dot-escaped */
 } ;
 
 struct hash_entry_s {
@@ -117,6 +124,7 @@ static bool artFreeContents (Article art) ;  /* Tell the Article to release
                                                 its contents buffer if
                                                 possible. */
 
+static void artUnmap (Article art, int size) ; /* munmap an mmap()ed article */
 
 
   /*
@@ -141,7 +149,7 @@ static void hashValidateTable (void) ;
 
 
   /*
-   * Private date
+   * Private data
    */
 
 
@@ -212,6 +220,8 @@ Article newArticle (const char *filename, const char *msgid)
       hashTable = ALLOC (HashEntry, TABLE_SIZE) ;
       ASSERT (hashTable != NULL) ;
       
+      addPointerFreedOnExit ((char *)hashTable) ;
+
       for (i = 0 ; i < TABLE_SIZE ; i++)
         hashTable [i] = NULL ;
 
@@ -235,9 +245,11 @@ Article newArticle (const char *filename, const char *msgid)
       strcpy (newArt->msgid,msgid) ;
       
       newArt->contents = NULL ;
+      newArt->mMapping = NULL ;
       newArt->refCount = 1 ;
       newArt->loggedMissing = false ;
       newArt->articleOk = true ;
+      newArt->inWireFormat = false ;
       
       dprintf (3,"Adding a new article(%p): %s\n", newArt, msgid) ;
       
@@ -252,7 +264,7 @@ Article newArticle (const char *filename, const char *msgid)
         syslog (LOG_ERR, DOUBLE_NAME, filename, newArt->fname) ;
       
       newArt->refCount++ ;
-      dprintf (2,"Reusing exiting article for %s\nx",msgid) ;
+      dprintf (2,"Reusing existing article for %s\nx",msgid) ;
     }
   
   return newArt ;
@@ -278,7 +290,10 @@ void delArticle (Article article)
 
       if (article->contents != NULL)
         {
-          bytesInUse -= bufferDataSize (article->contents) ;
+          if (article->mMapping)
+            artUnmap(article, bufferSize(article->contents)) ;
+          else
+            bytesInUse -= bufferDataSize (article->contents) ;
       
           if (article->nntpBuffers != NULL)
             freeBufferArray (article->nntpBuffers) ;
@@ -522,6 +537,18 @@ static Buffer artGetContents (Article article)
 }
 
 
+static void artUnmap (Article article, int size)
+{
+
+#ifdef HAVE_MMAP
+  if (munmap(article->mMapping, size) < 0)
+    syslog (LOG_NOTICE, "munmap %s: %m", article->fname) ;
+#endif
+
+  article->mMapping = NULL ;
+}
+
+
 
 static void logArticleStats (TimeoutId id, void *data)
 {
@@ -540,11 +567,17 @@ static void logArticleStats (TimeoutId id, void *data)
 
 
   /* do the actual read of the article off disk into a Buffer that is stored
-     in the Article object. */
+     in the Article object. The Article will end up with its contents field
+     having a buffer with the article data in it. This buffer may be
+     holding a mmapped pointer, or it may be simply a regular buffer with
+     the data read off disk into it. In the regular buffer case the
+     contents may be copied around after reading to insert a carriage
+     return before each newline. */
 
 static bool fillContents (Article article)
 {
   int fd ;
+  register char *p;
   struct stat buf ;
   static bool maxLimitNotified ;
 
@@ -579,113 +612,171 @@ static bool fillContents (Article article)
     }
   else
     {
-      char *buffer ;
+      char *buffer = NULL ;
       int amt = 0 ;
       size_t idx = 0, amtToRead = (size_t) buf.st_size ;
-      size_t newBufferSize ;
+      size_t newBufferSize = (size_t) buf.st_size ;
       HashEntry h ;
 
-      newBufferSize = (size_t) buf.st_size ;
-
-      if (bitFiddleContents)
+      if (useMMap)
         {
-            /* an estimate to give some room for nntpPrepareBuffer to use. */
-          newBufferSize *= (1.0 + (1.0 / avgCharsPerLine)) ;
-          newBufferSize ++ ;
-        }
+          /* if HAVE_MMAP is not defined, useMMap should never be
+             true, so we don't have to worry about other effects
+             of this block.  We only have to make the function compile. */
 
-        /* if we're going over the limit try to free up some older article's
-           contents.  */
-      if (amtToRead + bytesInUse > maxBytesInUse)
-        {
-          for (h = chronList ; h != NULL ; h = h->nextTime)
-            {
-              if (artFreeContents (h->article))
-                if (amtToRead + bytesInUse <= maxBytesInUse)
-                  break ;
-            }
-        }
+#ifdef HAVE_MMAP
+          article->mMapping = mmap((caddr_t) 0, (size_t) buf.st_size,
+                          PROT_READ, MAP_SHARED, fd, 0);
+#endif
 
-        /* we we couldn't get below, then log it (one time only) */
-      if ((amtToRead + bytesInUse) > maxBytesInUse &&
-          maxLimitNotified == false)
-        {
-          maxLimitNotified = true ;
-          syslog (LOG_NOTICE,MAX_BYTES_LIMIT,maxBytesInUse,
-                  amtToRead + bytesInUse) ;
-        }
-
-      if ((article->contents = newBuffer (newBufferSize)) == NULL)
-        amtToRead = 0 ;
-      else
-        buffer = bufferBase (article->contents) ;
-
-      bytesInUse += buf.st_size ;
-      byteTotal += buf.st_size ;
-
-      while (amtToRead > 0)
-        {
-          if ((amt = read (fd, buffer + idx,amtToRead)) <= 0)
-            {
-              syslog (LOG_ERR,BAD_ART_READ, article->fname) ;
-              bytesInUse -= buf.st_size ;
-              byteTotal -= buf.st_size ;
-              amtToRead = 0 ;
-
-              delBuffer (article->contents) ;
-              article->contents = NULL ;
+          if (article->mMapping == (caddr_t) -1)
+            {                   /* dunno, but revert to plain reading */
+              article->mMapping = NULL ;
+              syslog (LOG_NOTICE, MMAP_FAILURE, article->fname) ;
             }
           else
             {
-              idx += amt ;
-              amtToRead -= amt ;
+              article->contents = newBufferByCharP((char *)article->mMapping,
+                                                   (size_t) buf.st_size,
+                                                   (size_t) buf.st_size);
+              buffer = bufferBase (article->contents) ;
+              if ((p = strchr(buffer, '\n')) == NULL)
+                {                  
+                  article->articleOk = false;
+                  delBuffer (article->contents) ;
+                  article->contents = NULL ;
+                  syslog (LOG_NOTICE, MUNGED_ARTICLE, article->fname) ;
+                }
+              else if (p[-1] == '\r')
+                article->inWireFormat = true ;
+              else if (bitFiddleContents)
+                {
+                  /* we need to copy the contents into a buffer below */
+                  delBuffer (article->contents) ;
+                  article->contents = NULL ;
+                }
             }
         }
 
-      if (article->contents != NULL)
+      if (article->contents == NULL && article->articleOk)
         {
-          bufferSetDataSize (article->contents, (size_t) buf.st_size) ;
-          
           if (bitFiddleContents)
             {
-              if ( nntpPrepareBuffer (article->contents) )
+              /* an estimate to give some room for nntpPrepareBuffer to use. */
+              newBufferSize *= (1.0 + (1.0 / avgCharsPerLine)) ;
+              newBufferSize ++ ;
+            }
+          
+          /* if we're going over the limit try to free up some older article's
+             contents.  */
+          if (amtToRead + bytesInUse > maxBytesInUse)
+            {
+              for (h = chronList ; h != NULL ; h = h->nextTime)
                 {
-                  size_t diff = (bufferDataSize (article->contents) -
-                                 buf.st_size) ;
-                  
-                  if (UINT_MAX - diff <= preparedBytes)
-                    {
-                      dprintf (2,"Newline ratio so far: %02.2f\n",
-                               preparedBytes / preparedNewlines) ;
-                      syslog (LOG_NOTICE,PREPARED_NEWLINES,
-                              ((double) preparedBytes)/preparedNewlines,
-                              preparedBytes,preparedNewlines) ;
-                      preparedBytes = 0 ;
-                      preparedNewlines = 0 ;
-                      rolledOver = true ;
-                    }
-
-                  preparedBytes += buf.st_size ;
-                  preparedNewlines += diff ;
-                  bytesInUse += diff ;
-                  byteTotal += diff ;
-
-                  if (preparedBytes > (1024 * 1024))
-                    {
-                      avgCharsPerLine =
-                        ((double) preparedBytes) / preparedNewlines ;
-                      avgCharsPerLine++ ;
-                    }
+                  if (artFreeContents (h->article))
+                    if (amtToRead + bytesInUse <= maxBytesInUse)
+                      break ;
                 }
-              else
+            }
+
+          /* we we couldn't get below, then log it (one time only) */
+          if ((amtToRead + bytesInUse) > maxBytesInUse &&
+              maxLimitNotified == false)
+            {
+              maxLimitNotified = true ;
+              syslog (LOG_NOTICE,MAX_BYTES_LIMIT,maxBytesInUse,
+                      amtToRead + bytesInUse) ;
+            }
+
+          if ((article->contents = newBuffer (newBufferSize)) == NULL)
+            amtToRead = 0 ;
+          else
+            {
+              buffer = bufferBase (article->contents) ;
+              bytesInUse += buf.st_size ;
+              byteTotal += buf.st_size ;
+            }
+
+          if (article->mMapping && buffer != NULL)
+            {               
+              (void)memcpy(buffer, (char *) article->mMapping,
+                           (int) buf.st_size);
+              artUnmap(article, (int) buf.st_size) ;
+              amtToRead = 0;
+            }
+
+          while (amtToRead > 0)
+            {
+              if ((amt = read (fd, buffer + idx,amtToRead)) <= 0)
                 {
-                  syslog (LOG_ERR,PREPARE_FAILED) ;
+                  syslog (LOG_ERR,BAD_ART_READ, article->fname) ;
                   bytesInUse -= buf.st_size ;
                   byteTotal -= buf.st_size ;
+                  amtToRead = 0 ;
 
                   delBuffer (article->contents) ;
                   article->contents = NULL ;
                 }
+              else
+                {
+                  idx += amt ;
+                  amtToRead -= amt ;
+                }
+            }
+
+          if (article->contents != NULL)
+            {
+              bufferSetDataSize (article->contents, (size_t) buf.st_size) ;
+
+              if ((p = strchr(buffer, '\n')) == NULL)
+                {                  
+                  article->articleOk = false;
+                  syslog (LOG_NOTICE, MUNGED_ARTICLE, article->fname) ;
+                }
+              else if (p[-1] == '\r')
+                {
+                  article->inWireFormat = true ;
+                }
+              else if (bitFiddleContents)
+                if ( nntpPrepareBuffer (article->contents) )
+                  {
+                    size_t diff =
+                      (bufferDataSize (article->contents) - buf.st_size) ;
+
+                    if (UINT_MAX - diff <= preparedBytes)
+                      {
+                        dprintf (2,"Newline ratio so far: %02.2f\n",
+                                 ((double) preparedBytes / preparedNewlines)) ;
+                        syslog (LOG_NOTICE,PREPARED_NEWLINES,
+                                ((double) preparedBytes)/preparedNewlines,
+                                preparedBytes,preparedNewlines) ;
+                        preparedBytes = 0 ;
+                        preparedNewlines = 0 ;
+                        rolledOver = true ;
+                      }
+
+                    preparedBytes += buf.st_size ;
+                    preparedNewlines += diff ;
+                    bytesInUse += diff ;
+                    byteTotal += diff ;
+
+                    if (preparedBytes > (1024 * 1024))
+                      {
+                        avgCharsPerLine =
+                          ((double) preparedBytes) / preparedNewlines ;
+                        avgCharsPerLine++ ;
+                      }
+                    article->inWireFormat = true ;
+                  }
+                else
+                  {
+                    syslog (LOG_ERR,PREPARE_FAILED) ;
+                    bytesInUse -= buf.st_size ;
+                    byteTotal -= buf.st_size ;
+
+                    delBuffer (article->contents) ;
+                    article->contents = NULL ;
+                  }
             }
         }
     }
@@ -742,9 +833,15 @@ static bool prepareArticleForNNTP (Article article)
       dotBuffer = newBufferByCharP (".\r\n",3,3) ;
       dotFirstBuffer = newBufferByCharP ("\r\n.",3,3) ;
       crlfBuffer = newBufferByCharP ("\r\n",2,2) ;
+      addPointerFreedOnExit ((char *)dotBuffer) ;
+      addPointerFreedOnExit ((char *)dotFirstBuffer) ;
+      addPointerFreedOnExit ((char *)crlfBuffer) ;
     }
-      
-  if (!bitFiddleContents)
+
+  /* overlay a set of buffers on top of the articles contents buffer. This
+     is a real speed loss at the moment, so by default it's disabled (by
+     calling artBitFiddleContents(true) in main(). */
+  if (article->inWireFormat == false)
     {
       end = bufferBase (contents) ;
       do 
@@ -777,6 +874,7 @@ static bool prepareArticleForNNTP (Article article)
     {
         /* we already fixed the contents up when we read in the article */
       nntpBuffs = ALLOC (Buffer, 3) ;
+      ASSERT (nntpBuffs != NULL) ;
       nntpBuffs [0] = newBufferByCharP (bufferBase (contents),
                                         bufferDataSize (contents),
                                         bufferDataSize (contents)) ;
@@ -799,21 +897,21 @@ static bool artFreeContents (Article art)
   if (art->contents == NULL)
     return false ;
 
-  if (art->nntpBuffers != NULL && bufferRefCount (art->nntpBuffers[0]) > 1)
-    return false ;
-
   if (art->nntpBuffers != NULL)
-    {
-      ASSERT (art->contents != NULL) ;
-      
-      freeBufferArray (art->nntpBuffers) ;    
-      art->nntpBuffers = NULL ;
-    }
+    if (bufferRefCount (art->nntpBuffers[0]) > 1)
+      return false ;
+    else
+      {
+        freeBufferArray (art->nntpBuffers) ;    
+        art->nntpBuffers = NULL ;
+      }
 
-  if (art->contents != NULL)
-    ASSERT (bufferRefCount (art->contents) == 1) ;
+  ASSERT (bufferRefCount (art->contents) == 1) ;
 
-  bytesInUse -= bufferDataSize (art->contents) ;
+  if (art->mMapping)
+    artUnmap(art, bufferSize(art->contents)) ;
+  else
+    bytesInUse -= bufferDataSize (art->contents) ;
   
   delBuffer (art->contents) ;
 
