@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:08:24 1995
  * Project:     INN (innfeed)
  * File:        endpoint.c
- * RCSId:       $Id: endpoint.c,v 1.17 1996/12/07 02:28:42 brister Exp $
+ * RCSId:       $Id: endpoint.c,v 1.21 1997/01/13 00:34:53 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -38,7 +38,7 @@
  */
 
 #if ! defined (lint)
-static char rcsid [] = "$Id: endpoint.c,v 1.17 1996/12/07 02:28:42 brister Exp $" ;
+static char rcsid [] = "$Id: endpoint.c,v 1.21 1997/01/13 00:34:53 brister Exp $" ;
 static void use_rcsid (const char *rId) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rId) ;
 } 
@@ -58,6 +58,8 @@ static void use_rcsid (const char *rId) {   /* Never called */
 #include <errno.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/select.h>
 
 #if defined (DO_NEED_TIME)
 #include <time.h>
@@ -152,11 +154,11 @@ typedef struct timerqelem_s
 int debugWrites ;
 void (*gPrintInfo) (void) ;
 
-extern char *InputFile ;
-extern void addPointerFreedOnExit (char *pointerToFree) ;
+extern const char *InputFile ;
 
-static int sigFlag ;
+static bool sigFlag ;
 static EndPoint mainEndPoint ;
+static bool mainEpIsReg = false ;
 
 time_t  PrivateTime;
 
@@ -544,10 +546,10 @@ void Run ()
         die ("No fd's ready and no timeouts") ;
       else if (sval < 0 && errno == EINTR)
         {
-          if (sigFlag && gPrintInfo != NULL)
+          if (sigFlag == true && gPrintInfo != NULL)
             {
               gPrintInfo () ;
-              sigFlag = 0 ;
+              sigFlag = false ;
             }
         }
       else if (sval < 0) 
@@ -561,10 +563,10 @@ void Run ()
           int readyCount = sval ;
           int timesThrough = 0 ;
           
-          if (sigFlag && gPrintInfo != NULL)
+          if (sigFlag == true && gPrintInfo != NULL)
             {
               gPrintInfo () ;
-              sigFlag = 0 ;
+              sigFlag = false ;
             }
           
           for (idx = 0 ; idx < priorityCount ; idx++)
@@ -579,43 +581,60 @@ void Run ()
                   timesThrough++ ;
                   
                   /* Every SELECT_RATIO times around this loop we check to
-                     see if the mainEndPoint fd is ready to read. If so we
-                     process it and do the current endpoint next time
-                     around */
-                  if (idx > 0 && mainEndPoint != NULL
-                      && ep != mainEndPoint &&
-                      (timesThrough % SELECT_RATIO) == 0)
+                     see if the mainEndPoint fd is ready to read or
+                     write. If so we process it and do the current endpoint
+                     next time around. */
+                  if (((timesThrough % SELECT_RATIO) == 0) &&
+                      ep != mainEndPoint && mainEndPoint != NULL &&
+                      !mainEpIsReg)
                     {
-                      fd_set trSet ;
+                      fd_set trSet, twSet ;
                       struct timeval tw ;
+                      int checkRead = FD_ISSET (mainEndPoint->myFd,&rdSet) ;
+                      int checkWrite = FD_ISSET (mainEndPoint->myFd,&wrSet) ;
 
-                      fd = mainEndPoint->myFd ;
+                      if (checkRead || checkWrite) 
+                        {
+                          fd = mainEndPoint->myFd ;
+
+                          tw.tv_sec = tw.tv_usec = 0 ;
+                          memset (&trSet,0,sizeof (trSet)) ;
+                          memset (&twSet,0,sizeof (twSet)) ;
                       
-                      tw.tv_sec = tw.tv_usec = 0 ;
-                      memset (&trSet,0,sizeof (trSet)) ;
-                      FD_SET (fd,&trSet) ;
+                          if (checkRead)
+                            FD_SET (fd,&trSet) ;
+                          if (checkWrite)
+                            FD_SET (fd,&twSet) ;
 
-                      sval = select (fd + 1, &trSet,0,0,&tw) ;
+                          sval = select (fd + 1,&trSet,&twSet,0,&tw) ;
 
-                      if (sval > 0)
-                        {
-                          idx-- ;
-                          ep = mainEndPoint ;
-                          FD_SET (fd,&rSet) ;
-                          FD_SET (fd,&rdSet) ;
-                          readyCount++ ;
-                          specialCheck = true ;
+                          if (sval > 0)
+                            {
+                              idx-- ;
+                              ep = mainEndPoint ;
+                              specialCheck = true ;
+                              if (checkRead && FD_ISSET (fd,&trSet))
+                                {
+                                  FD_SET (fd,&rSet) ;
+                                  readyCount++ ;
+                                }
+                              if (checkWrite && FD_ISSET (fd,&twSet))
+                                {
+                                  FD_SET (fd,&wSet) ;
+                                  readyCount++ ;
+                                }
+                            }
+                          else if (sval < 0)
+                            {
+                              syslog (LOG_ERR,BAD_SELECT,sval) ;
+                              stopRun () ;
+                              return ;
+                            }
+                          else
+                            fd = ep->myFd ; /* back to original fd. */
                         }
-                      else if (sval < 0)
-                        {
-                          syslog (LOG_ERR,BAD_SELECT,sval) ;
-                          stopRun () ;
-                          return ;
-                        }
-                      else
-                        fd = ep->myFd ; /* back to original fd. */
                     }
-                  
+                      
                   ep->timesSelected += (FD_ISSET (fd,&wrSet) ? 1 : 0) ;
                   ep->timesSelected += (FD_ISSET (fd,&rdSet) ? 1 : 0) ;
   
@@ -648,12 +667,12 @@ void Run ()
                         }
                     }
 
-                  if (specialCheck)
-                    continue ;
-                  
                   /* get it again as the read callback may have deleted the */
                   /* endpoint */
-                  ep = priorityList [idx] ;
+                  if (specialCheck)
+                    ep = mainEndPoint ;
+                  else
+                    ep = priorityList [idx] ;
                   
                   if (readyCount > 0 && ep != NULL && FD_ISSET (fd,&wSet))
                     {
@@ -683,7 +702,10 @@ void Run ()
                   
                   /* get it again as the write callback may have deleted the */
                   /* endpoint */
-                  ep = priorityList [idx] ;
+                  if (specialCheck)
+                    ep = mainEndPoint ;
+                  else
+                    ep = priorityList [idx] ;
 
                   if (readyCount > 0 && ep != NULL && FD_ISSET (fd,&eSet))
                     doExcept (ep) ;
@@ -756,7 +778,15 @@ bool writeIsPending (EndPoint endp)
 
 void setMainEndPoint (EndPoint endp)
 {
+  struct stat buf ;
+
   mainEndPoint = endp ;
+  if (endp->myFd >= 0 && fstat (endp->myFd,&buf) < 0)
+    syslog (LOG_ERR,"Can't fstat mainEndPoint fd (%d): %m", endp->myFd) ;
+  else if (endp->myFd < 0)
+    syslog (LOG_ERR,"Negative fd for mainEndPoint???") ;
+  else
+    mainEpIsReg = S_ISREG(buf.st_mode) ;
 }
 
 void freeTimeoutQueue (void)
@@ -1038,7 +1068,7 @@ static void endPointPrint (EndPoint ep, FILE *fp)
 
 static void intHandler (int s)
 {
-  sigFlag = 1 ;
+  sigFlag = true ;
   signal (s, intHandler) ;
 }
 
