@@ -4,7 +4,7 @@
  * Start Date:  Thu Dec 28 17:29:05 1995
  * Project:     INN (innfeed)
  * File:        host.c
- * RCSId:       $Id: host.c,v 1.22 1996/12/07 01:33:06 brister Exp $
+ * RCSId:       $Id: host.c,v 1.24 1997/01/13 00:35:14 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: host.c,v 1.22 1996/12/07 01:33:06 brister Exp $" ;
+static const char *rcsid = "$Id: host.c,v 1.24 1997/01/13 00:35:14 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -154,6 +154,14 @@ struct host_s
 };
 
 
+/* These numbers are as above, but for all hosts over
+   the life of the process. */
+long procArtsOffered ;        
+long procArtsAccepted ;
+long procArtsNotWanted ;
+long procArtsRejected ;
+long procArtsDeferred ;
+long procArtsMissing ;
 
 
 static u_int defaultArticleTimeout ;
@@ -229,6 +237,8 @@ static time_t start ;
 static char startTime [30] ;    /* for ctime(3) */
 static pid_t myPid ;
 
+static char *statusFile = NULL ;
+
 /*******************************************************************/
 /*                  PUBLIC FUNCTIONS                               */
 /*******************************************************************/
@@ -248,9 +258,13 @@ void configHosts (bool talkSelf)
   double lowFilter, highFilter ;
   bool streaming ;
   u_short portNum ;
+  char path [PATH_MAX] ;
 
   name = ipName = NULL ;
 
+  closeDroppedArticleFile () ;
+  openDroppedArticleFile () ;
+
   if ( strlen (configFile) == 0 || !fileExistsP (configFile) )
     {
       syslog (LOG_ERR, NOSUCH_CONFIG, configFile) ;
@@ -275,7 +289,7 @@ void configHosts (bool talkSelf)
       h = findHostByName (name) ;
       if ( h != NULL )
         {
-	  int i ;
+	  u_int i ;
 
 	  if (strcmp(h->ipName, ipName) != 0)
 	    {
@@ -1125,6 +1139,13 @@ bool hostCxnGone (Host host, Connection cxn)
       hostsLeft = listenerHostGone (host->listener, host) ;
       delHost (host) ;
 
+      if (hostsLeft == 0)
+        syslog (LOG_NOTICE,PROCESS_FINAL_STATS,
+                (long) (now - start),
+                procArtsOffered, procArtsAccepted,
+                procArtsNotWanted,procArtsRejected,
+                procArtsMissing) ;
+      
       /* return true if that was the last host */
       return hostsLeft == 0 ;
     }
@@ -1147,7 +1168,8 @@ void hostArticleOffered (Host host, Connection cxn)
   (void) cxn ;                  /* keep lint happy. */
   
   host->artsOffered++ ;
-  host->gArtsOffered ++ ;
+  host->gArtsOffered++ ;
+  procArtsOffered++ ;
 }
 
 
@@ -1168,6 +1190,7 @@ void hostArticleAccepted (Host host, Connection cxn, Article article)
   
   host->artsAccepted++ ;
   host->gArtsAccepted++ ;
+  procArtsAccepted++ ;
 
   /* host has two references to the article here... the parameter `article'
      and the queue */
@@ -1196,6 +1219,7 @@ void hostArticleNotWanted (Host host, Connection cxn, Article article)
   
   host->artsNotWanted++ ;
   host->gArtsNotWanted++ ;
+  procArtsNotWanted++ ;
   
   
   /* host has two references to the article here... `article' and the
@@ -1225,6 +1249,7 @@ void hostArticleRejected (Host host, Connection cxn, Article article)
   
   host->artsRejected++ ;
   host->gArtsRejected++ ;
+  procArtsRejected++ ;
 
   /* host has two references to the article here... `article' and the queue */
 
@@ -1247,6 +1272,7 @@ void hostArticleDeferred (Host host, Connection cxn, Article article)
 {
   host->artsDeferred++ ;
   host->gArtsDeferred++ ;
+  procArtsDeferred++ ;
 
   host->backlogSpooled++ ;
 
@@ -1301,6 +1327,7 @@ void hostArticleIsMissing (Host host, Connection cxn, Article article)
     
   host->artsMissing++ ;
   host->gArtsMissing++ ;
+  procArtsMissing++ ;
 
   /* host has two references to the article here... `article' and the
      queue */
@@ -1464,6 +1491,27 @@ void hostLogNoCheckMode (Host host, bool on)
 
 
 
+void hostSetStatusFile (const char *filename)
+{
+  if (statusFile != NULL)
+    die ("Can't set status file name twice\n") ;
+  else if (filename == NULL)
+    die ("Can't set status file name with a NULL filename\n") ;
+  else if (*filename == NULL)
+    die ("Can't set status file name with a empty string\n") ;
+
+  if (*filename == '/') 
+    statusFile = strdup (filename) ;
+  else
+    {
+      const char *tapeDir = getTapeDirectory() ;
+      
+      statusFile = malloc (strlen (tapeDir) + strlen (filename) + 2) ;
+      sprintf (statusFile,"%s/%s",tapeDir,filename) ;
+    }
+}
+
+
 
 /**********************************************************************/
 /**                      PRIVATE FUNCTIONS                           **/
@@ -1899,7 +1947,6 @@ static void hostLogStats (Host host, bool final)
 extern char *versionInfo ;
 static void hostLogStatus (void)
 {
-  char path [MAXPATHLEN] ;
   FILE *fp = NULL ;
   Host h ;
   bool anyToLog = false ;
@@ -1916,14 +1963,16 @@ static void hostLogStatus (void)
 
   if (!anyToLog)
     return ;
+
+  if (statusFile == NULL)
+    hostSetStatusFile (INNFEED_STATUS) ;
   
-  sprintf (path,"%s/%s",getTapeDirectory (),"innfeed.status") ;
-  if ((fp = fopen (path,"w")) == NULL)
+  if ((fp = fopen (statusFile,"w")) == NULL)
     {
       static bool logged ;
 
       if ( !logged )
-        syslog (LOG_ERR,NO_STATUS,path) ;
+        syslog (LOG_ERR,NO_STATUS,statusFile) ;
       logged = true ;
     }
   else
@@ -1940,7 +1989,7 @@ static void hostLogStatus (void)
 
       fprintf (fp,"%s\npid %d started %s\nUpdated: %s",
                versionInfo,(int) myPid,startTime,timeString) ;
-      fprintf (fp," (peers: %d active: %d sleeping: %d)\n\n",
+      fprintf (fp," (peers: %d active-cxns: %d sleeping-cxns: %d)\n\n",
                peerNum, actConn, slpConn) ;
                
       fprintf (fp,"Default configuration parameters:\n") ;
@@ -1960,6 +2009,20 @@ static void hostLogStatus (void)
       fprintf (fp,"      off threshold: %-2.1f%%\n\n",
 	       (defaultLowFilter * 10)) ;
 
+
+#if 1
+      fprintf (fp,"global (process)\n") ;
+      
+      fprintf (fp, "   seconds: %-7ld\n", (long) (now - start)) ;
+      fprintf (fp, "   offered: %-7ld\n", procArtsOffered) ;
+      fprintf (fp, "  accepted: %-7ld\n", procArtsAccepted) ;
+      fprintf (fp, "   refused: %-7ld\n", procArtsNotWanted) ;
+      fprintf (fp, "  rejected: %-7ld\n", procArtsRejected) ;
+      fprintf (fp, "   missing: %-7ld\n", procArtsMissing) ;
+      fprintf (fp, "  deferred: %-7ld\n", procArtsDeferred) ;
+      fprintf (fp, "\n");
+#endif
+      
       for (h = gHostList ; h != NULL ; h = h->next)
         hostPrintStatus (h,fp) ;
 
