@@ -4,7 +4,7 @@
  * Start Date:  Mon, 15 Jan 1996 17:31:58 +1100
  * Project:     INN -- innfeed
  * File:        main.c
- * RCSId:       $Id: main.c,v 1.19 1996/12/07 01:38:53 brister Exp $
+ * RCSId:       $Id: main.c,v 1.23 1997/01/13 00:40:41 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: main.c,v 1.19 1996/12/07 01:38:53 brister Exp $" ;
+static const char *rcsid = "$Id: main.c,v 1.23 1997/01/13 00:40:41 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -82,11 +82,12 @@ static void use_rcsid (const char *rid) {   /* Never called */
 
 /* exports */
 extern int debugWrites ;
-int sigFlag ;
-char *InputFile ;
-const char *configFile = CONFIG_FILE ;
-bool RollInputFile ;
-char *pidFile = PID_FILE ;
+bool sigFlag = false ;
+const char *InputFile ;
+const char *configFile = NULL ;
+bool RollInputFile = false ;
+const char *pidFile = PID_FILE ;
+bool useMMap = false ;
 
 /* imports */
 extern char *versionInfo ;
@@ -100,7 +101,7 @@ extern void openInputFile (void);
 /* privates */
 static bool talkToSelf ;
 static char *logFile ;
-static char *newsspool = NEWSSPOOL;
+static const char *newsspool = NEWSSPOOL;
 
 static void sigalrm (int sig) ;
 static void sigchld (int sig) ;
@@ -116,12 +117,16 @@ int main (int argc, char **argv)
 {
   EndPoint ep ;
   InnListener listener ;
-  int optVal ;
+  int optVal, fd ;
   const char *subProgram = NULL ;
   bool seenV = false ;
   bool dynamicPeers = false ;
   time_t now = theTime() ;
   char dateString [30] ;
+  char *sopt = NULL ;
+  char *copt = NULL ;
+  char *lopt = NULL ;
+  int lim ;
 
   strcpy (dateString,ctime(&now)) ;
   dateString [24] = '\0' ;
@@ -133,9 +138,18 @@ int main (int argc, char **argv)
 
   gPrintInfo = gprintinfo ;
 
+#if defined (HAVE_MMAP)
+  useMMap = true ;
+#else
+  useMMap = false ;
+#endif
+
+  /* always turn on copying into memory unless mmapping and the article
+     data being in NNTP format. */
+  artBitFiddleContents (true) ;
+
+#define OPT_STRING "a:b:c:d:e:hl:mMo:p:S:s:vxyz"
 
-#define OPT_STRING "zymvhxb:l:c:d:s:o:p:a:"
-  
   while ((optVal = getopt (argc,argv,OPT_STRING)) != EOF)
     {
       switch (optVal) 
@@ -143,31 +157,42 @@ int main (int argc, char **argv)
           case 'a':
             newsspool = optarg ;
             break ;
-            
+
+          case 'b':
+            if ( !isDirectory (optarg) )
+              logAndExit (1,"Not a directory: %s\n",optarg) ;
+            setTapeDirectory (optarg) ;
+            break ;
+
+          case 'c':
+            copt = optarg ;
+            break ;
+
           case 'd':
             loggingLevel = atoi (optarg) ;
             break ;
 
-          case 'z':
-            hostLogConnectionStats (true) ;
+          case 'e':
+            lim = atoi (optarg) ;
+            if (lim <= 0)
+              {
+                fprintf (stderr,"Illegal value for -e option\n") ;
+                usage (1) ;
+              }
+            setOutputSizeLimit (atoi (optarg)) ;
             break ;
             
-          case 'x':
-            talkToSelf = true ;
-            break ;
-            
-          case 's':
-            subProgram = optarg ;
-            break ;
-
-          case 'b':
-            setTapeDirectory (optarg) ;
-            break ;
+          case 'h':
+            usage (0) ;
 
           case 'l':
-            logFile = optarg ;
+            lopt = optarg ;
             break ;
 
+          case 'M':
+            useMMap = false ;
+            break ;
+            
           case 'm':
             artLogMissingArticles (true) ;
             break ;
@@ -180,23 +205,32 @@ int main (int argc, char **argv)
             pidFile = optarg ;
             break ;
 
-          case 'c':
-            configFile = optarg ;
+          case 's':
+            subProgram = optarg ;
             break ;
 
-          case 'h':
-            usage (0) ;
+          case 'S':
+            sopt = optarg ;
+            break ;
 
           case 'v':
             seenV = true ;
             break ;
 
+          case 'x':
+            talkToSelf = true ;
+            break ;
+            
           case 'y':
             dynamicPeers = true ;
             break ;
-	
+
+          case 'z':
+            hostLogConnectionStats (true) ;
+            break ;
+            
           default:
-             	usage (1) ;	
+            usage (1) ;	
         }
     }
 
@@ -208,15 +242,56 @@ int main (int argc, char **argv)
   else if (argc == 1)
     InputFile = *argv;
 
+  if (seenV)
+    {
+      warn ("%s version: %s\n",program, versionInfo) ;
+      exit (0) ;
+    }
+
+  /* make sure we have valid fds 0, 1 & 2 so it is not taken by
+     something else, probably openlog().  fd 0 will be freopen()ed on the
+     inputFile, the subProgram, or ourself.  fd 1 and fd 2 will
+     be freopen()ed on the log file (or will stay pointed at /dev/null).
+
+     without doing this, if the descriptors were closed then the
+     freopen calls on some systems (like BSDI 2.1) will really close
+     whatever has aquired the stdio descriptors, such as the socket
+     to syslogd.
+
+     XXX possible problems:  what if fd 0 is closed but no inputFile,
+     XXX subProgram or talkToSelf is true?  it will not be freopen()ed, so
+     XXX innfeed won't have any fresh data (besides, fd 0 is only writable
+     XXX here).  perhaps a warning should be issued.
+   */
+  do
+    {
+      fd = open("/dev/null", O_WRONLY);
+      switch (fd)
+        {
+        case -1:
+          logAndExit (1,"open(\"/dev/null\", O_WRONLY): %s", strerror (errno));
+          break;
+        case 0:
+        case 1:
+        case 2:
+          /* good, we saved an fd from being trounced */
+          break;
+        default:
+          close(fd);
+        }
+    } while (fd < 2);
+
+  openlog (program,(int)(L_OPENLOG_FLAGS|LOG_PID),LOG_NEWS) ;
+
+  if (talkToSelf)
+    setOutputSizeLimit (0) ;
+
   if (subProgram == NULL && talkToSelf == false)
     {
       struct stat buf ;
 
       if (fstat (0,&buf) < 0)
-        {
-          syslog (LOG_ERR,FSTAT_FAILURE,"stdin") ;
-          exit (1) ;
-        }
+        logAndExit (1,FSTAT_FAILURE,"stdin") ;
       else if (S_ISREG (buf.st_mode))
           InputFile = "";
     }
@@ -224,28 +299,34 @@ int main (int argc, char **argv)
   if (subProgram != NULL && (talkToSelf == true || InputFile))
     {
       dprintf (0,"Cannot specify '-s' with '-x' or an input file\n") ;
+      syslog (LOG_ERR,"Incorrect arguments: '-s' with '-x' or an input file\n");
       usage (1) ;
     }
 
+  if (sopt != NULL)
+    hostSetStatusFile (sopt) ;
+
+  if (copt != NULL && *copt == '\0')
+    logAndExit (1,"Empty pathname for ``-c'' option") ;
+  else if (copt != NULL)
+    configFile = buildFilename (getTapeDirectory(),copt) ;
+  else
+    configFile = buildFilename (getTapeDirectory(),CONFIG_FILE) ;
+  
+  if (lopt != NULL && *lopt == '\0')
+    logAndExit (1,"empty pathanme for ``-l'' option") ;
+  else if (lopt != NULL)
+    logFile = buildFilename (getTapeDirectory(),lopt) ;
+  
   if (loggingLevel == 0 && fileExistsP (DEBUG_FILE))
     loggingLevel = 1 ;
 
-    /* XXX this need to be an option */
-  artBitFiddleContents (true) ; 
- 
   if (logFile == NULL && ! isatty ( fileno (stderr) ) )
     logFile = LOG_FILE ;
 
   if (logFile)
     openLogFile () ;
 
-  if (seenV)
-    {
-      warn ("%s version: %s\n",program, versionInfo) ;
-      exit (0) ;
-    }
-
-  openlog (program,(int)(L_OPENLOG_FLAGS|LOG_PID),LOG_NEWS) ;
   syslog (LOG_NOTICE,STARTING_PROGRAM,versionInfo,dateString) ;
 
   openfds = 4 ;                 /* stdin, stdout, stderr and syslog */
@@ -350,7 +431,7 @@ int main (int argc, char **argv)
   /* handle signal to reload config */
   signal (SIGHUP,sighup) ;
 
-  /* handle signal to roll log file */
+  /* handle signal to roll input file */
   signal (SIGALRM,sigalrm) ;
 
   /* we can increment and decrement logging levels by sending SIGUSR{1,2} */
@@ -365,37 +446,64 @@ int main (int argc, char **argv)
 
 static void usage (int val)
 {
-  fprintf (stderr,"usage: %s [ options ]\n\n",
+  fprintf (stderr,"usage: %s [ options ] [ file ]\n\n",
            program) ;
   fprintf (stderr,"Version: %s\n\n",versionInfo) ;
   fprintf (stderr,"Config file: %s\n",CONFIG_FILE) ;
   fprintf (stderr,"Backlog directory: %s\n",TAPE_DIRECTORY) ;
-  fprintf (stderr,"\n\nLegal options are:\n") ;
+  fprintf (stderr,"\nLegal options are:\n") ;
+  fprintf (stderr,"\t-a dir      Use the given directory as the top of the article spool\n") ;
+
+  fprintf (stderr,"\t-b dir      Use the given directory as the the storage\n");
+  fprintf (stderr,"\t            place for backlog files and lock files.\n");
+
+  fprintf (stderr,"\t-c file     Use the given file as the config file instead of the\n");
+  fprintf (stderr,"\t            default of %s\n",CONFIG_FILE);
+
   fprintf (stderr,"\t-d num      set the logging level to num (an integer).\n");
   fprintf (stderr,"\t            Larger value means more logging. 0 means no\n");
   fprintf (stderr,"\t            logging. The default is 0\n");
+
+  fprintf (stderr,"\t-e bytes    Keep the output backlog files to no bigger\n");
+  fprintf (stderr,"\t            than %.2f times this number\n",LIMIT_FUDGE);
+  
+  fprintf (stderr,"\t-h          print this message\n");
+
   fprintf (stderr,"\t-l file     redirect stderr and stdout to the given file.\n");
   fprintf (stderr,"\t            When run under INN they normally are redirected to\n");
   fprintf (stderr,"\t            /dev/null. This is needed if using '-d'.\n");
+
+  fprintf (stderr,"\t-m          Log information on all missing articles\n");
+
+  fprintf (stderr,"\t-M          Turn *off* use of mmap\n") ;
+#if ! defined (HAVE_MMAP)
+  fprintf (stderr,"\t            (a no-op as this excutable has been built without mmap support\n") ;
+#endif
+
+  fprintf (stderr,"\t-p file     Write the process id to the given file\n") ;
+  fprintf (stderr,"\t            instead of the default of %s\n",PID_FILE);
+  fprintf (stderr,"\t            A relative path is relative to the backlog directory\n") ;
+
   fprintf (stderr,"\t-s command  run the given command in a subprocess and use\n");
   fprintf (stderr,"\t            its output as article information instead of\n");
   fprintf (stderr,"\t            running under innd\n");
-  fprintf (stderr,"\t-x          Do not read any articles information off stdin,\n");
+
+  fprintf (stderr,"\t-S file     Use the give filename instead of innfeed.status\n") ;
+  fprintf (stderr,"\t            relative pathnames start from the backlog directory\n") ;
+
+  fprintf (stderr,"\t-v          print version information\n");
+
+  fprintf (stderr,"\t-x          Do not read any article information off stdin,\n");
   fprintf (stderr,"\t            but simply process backlog files and then exit\n");
   fprintf (stderr,"\t            when done\n");
-  fprintf (stderr,"\t-c file     Use the given file as the config file instead of the\n");
-  fprintf (stderr,"\t            default of %s\n",CONFIG_FILE);
+
+  fprintf (stderr,"\t-y          Add peers dynamically. If an unrecognized peername\n");
+  fprintf (stderr,"\t            is received from innd, then it is presumed to also\n");
+  fprintf (stderr,"\t            be the ip name and a new peer binding is set up\n");
+
   fprintf (stderr,"\t-z          have each of the connections issue their own stats\n");
   fprintf (stderr,"\t            whenever they close, or whenever their controller\n");
   fprintf (stderr,"\t            issues its own stats\n");
-  fprintf (stderr,"\t-y          Add peers dynamically. If an unrecognized peername\n");
-  fprintf (stderr,"\t            is received from innd, then it is presumed to als\n");
-  fprintf (stderr,"\t            be the ip name and a new peer binding is set up\n");
-  fprintf (stderr,"\t-m          Log information on all missing articles\n");
-  fprintf (stderr,"\t-v          print version information\n");
-  fprintf (stderr,"\t-h          print this message\n");
-  fprintf (stderr,"\t-b dir      Use the given directory as the the storage\n");
-  fprintf (stderr,"\t            place for backlog (tape) files and lock files.\n");
 
   exit (val) ;
 }
@@ -456,27 +564,38 @@ static void openLogFile ()
 {
   FILE *fpr ;
 
-  fpr = freopen (logFile,"a",stdout) ;
-  if (fpr != stdout)
-    die ("freopen (%s, \"a\", stdout): %s", logFile, strerror (errno)) ;
+  if (logFile)
+    {
+      fpr = freopen (logFile,"a",stdout) ;
+      if (fpr != stdout)
+        logAndExit (1,"freopen (%s, \"a\", stdout): %s",
+                    logFile, strerror (errno)) ;
 
-  fpr = freopen (logFile,"a",stderr) ;
-  if (fpr != stderr)
-    die ("freopen (%s, \"a\", stderr): %s", logFile, strerror (errno)) ;
+      fpr = freopen (logFile,"a",stderr) ;
+      if (fpr != stderr)
+        logAndExit (1,"freopen (%s, \"a\", stderr): %s",
+                    logFile, strerror (errno)) ;
 
 #if defined (DO_HAVE_SETBUFFER)
-setbuffer (stdout, NULL, 0) ;
-setbuffer (stderr, NULL, 0) ;
+      setbuffer (stdout, NULL, 0) ;
+      setbuffer (stderr, NULL, 0) ;
 #else
-setbuf (stdout, NULL) ;
-setbuf (stderr, NULL) ;
+      setbuf (stdout, NULL) ;
+      setbuf (stderr, NULL) ;
 #endif
+    }
 }
 
 static void writePidFile ()
 {
   FILE *F;
   int pid;
+  char path [PATH_MAX] ;
+
+  pidFile = buildFilename (getTapeDirectory(),pidFile) ;
+  
+  if (pidFile == NULL)
+    logAndExit (1,"NULL pidFile\n") ;
 
   /* Record our PID. */
   pid = getpid();
