@@ -4,7 +4,7 @@
  * Start Date:  Sun, 28 Jan 1996 18:37:49 +1100
  * Project:     INN -- innfeed
  * File:        msgs.h
- * RCSId:       $Id: msgs.h,v 1.19 1996/12/07 01:39:10 brister Exp $
+ * RCSId:       $Id: msgs.h,v 1.22 1997/01/13 00:43:08 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -51,7 +51,7 @@
 #define RESPONSE_READ_FAILED    "%s:%d cxnsleep can't read response : %m"
 #define BANNER_READ_FAILED      "%s:%d cxnsleep can't read banner : %m"
 #define SOCKET_CREATE_ERROR     "%s:%d cxnsleep can't create socket : %m"
-#define MODE_WRITE_PENDING	"%s:%s cxnsleep mode stream command still pending"
+#define MODE_WRITE_PENDING	"%s:%d cxnsleep mode stream command still pending"
 #define NOMSGID                 "%s:%d cxnsleep message-id missing in reponse code %d: %s"
 #define INVALID_MSGID           "%s:%d cxnsleep message-id invalid message-id in reponse code %d : %s"
 #define UNKNOWN_RESPONSE        "%s:%d cxnsleep response unknown : %d : %s"
@@ -66,7 +66,7 @@
 #define CONNECTION_FAILURE      "%s:%d cxnsleep connect : %m"
 #define IO_FAILED               "%s:%d cxnsleep i/o failed : %m"
 #define NO_TRANSFER_NNRPD	"%s:%d cxnsleep transfer permission denied"
-#define NO_TALK_NNRPD		"%s:%d cxnsleep no permission to talk"
+#define NO_TALK_NNRPD		"%s:%d cxnsleep no permission to talk: %s"
 #define CXN_BAD_STATE		"%s:%d cxnsleep connection in bad state: %s"
 #define CXN_STREAM_RESP		"%s:%d cxnsleep unexpected streaming response for non-streaming connection: %s"
 #define CXN_NONSTREAM_RESP	"%s:%d cxnsleep unexpected non-streaming response for streaming connection: %s"
@@ -96,6 +96,7 @@
 #define STREAMING_MODE_UNDO     "%s mode no-CHECK exited"
 
 #define REALLY_FINAL_STATS	"%s global seconds %ld offered %d accepted %d refused %d rejected %d missing %d"
+#define PROCESS_FINAL_STATS	"ME global seconds %ld offered %d accepted %d refused %d rejected %d missing %d"
 		/* key word in next two is "checkpoint" or "final" */
 #define HOST_STATS_MSG          "%s %s seconds %ld offered %d accepted %d refused %d rejected %d missing %d spooled %d"
 #define HOST_SPOOL_STATS        "%s %s seconds %ld spooled %d"
@@ -121,10 +122,23 @@
 #define DYNAMIC_PEER            "ME unconfigured peer %s added"
 #define UNKNOWN_PEER            "ME unconfigured peer %s"
 #define STOPPING_PROGRAM        "ME finishing at %s"
+#define NO_DROPPED_FILE		"ME cant open %s: loosing articles: %m"
+#define NO_NULL_FILE		"ME cant open /dev/null: %m"
+#define LOST_ARTICLE_COUNT	"ME lost %ld articles"
+#define DROPPED_ARTICLE_COUNT	"ME dropped %ld articles"
+#define DROPPED_LOCATION	"ME dropping articles into %s"
 
 #define L_BUFFER_EXPAND_ERROR	"ME error expanding input buffer"
 #define L_PREPARE_READ_FAILED	"ME error prepare read failed"
 
+#define SHRINK_TEMP_OPEN	"ME error opening temp shrink file %s: %m"
+#define SHRINK_SEEK		"ME error seeking to point %ld in %s"
+#define SHRINK_NONL		"ME no newline in shrinking file %s"
+#define SHRINK_WRITETMP		"ME fwrite failed to temp shrink file %s: %m"
+#define SHRINK_READ		"ME fread failed on file %s: %m"
+#define SHRINK_FREOPEN		"ME freopen on shrink file failed %s %m"
+#define SHRINK_WRITEORIG	"ME fwrite failed to shrink file %s %m"
+
 
 /* messages used in commander.c */
 
@@ -153,6 +167,8 @@
 #define ACTIVE_ARTICLES         "ME articles active %d bytes %d"
 #define ARTICLE_ALLOCS          "ME articles total %d bytes %d"
 #define MAX_BYTES_LIMIT         "ME exceeding maximum article byte limit: %d (max), %d (cur)"
+#define MMAP_FAILURE		"ME mmap failure %s: %m"
+#define MUNGED_ARTICLE		"ME munged article %s"
 
 
 /* main.c */
@@ -184,12 +200,12 @@
 #define NO_WRITE_LOCK_PID       "ME lock file pid-write: %m"
 #define BAD_PID                 "ME lock bad-pid info in %s : %s"
 #define LOCK_EXISTS		"ME lock in-use already: %s by pid %d"
-
+#define FILE_SHRUNK		"ME file %s shrunk from %ld to %ld"
 
 /* tape.c */
 
 #define TAPE_OPEN_FAILED        "ME tape open failed (%s) %s: %m"
-#define FILE_SHORT              "ME tape shrunk : %s %ld %ld"
+#define FILE_SHORT              "ME tape short : %s %ld %ld"
 #define NO_LOCK_TAPE            "ME lock failed for host: %s"
 #define TAPE_INPUT_ERROR        "ME ioerr on tape file: %s : %m"
 #define FCLOSE_FAILED           "ME ioerr fclose %s : %m"
@@ -200,6 +216,6 @@
 #define FSEEK_FAILED            "ME oserr fseek %s,%ld,SEEK_SET : %m"
 #define CHECKPOINT_OPEN         "ME oserr open checkpoint file: %s %m"
 #define BAD_CHECKPOINT          "ME internal bad data in checkpoint file: %s"
-
+#define CKPT_BNDRY		"ME internal checkpoint line boundary missed: %s %ld vs. %ld"
 #endif /* msgs_h__ */
 
