@@ -4,7 +4,7 @@
  * Start Date:  Sun Dec 31 18:52:04 1995
  * Project:     INN (innfeed)
  * File:        tape.c
- * RCSId:       $Id: tape.c,v 1.15 1996/11/20 18:02:44 brister Exp $
+ * RCSId:       $Id: tape.c,v 1.19 1997/01/13 00:54:29 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -40,7 +40,7 @@
 
  
 #if ! defined (lint)
-static const char *rcsid = "$Id: tape.c,v 1.15 1996/11/20 18:02:44 brister Exp $" ;
+static const char *rcsid = "$Id: tape.c,v 1.19 1997/01/13 00:54:29 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -85,27 +85,28 @@ typedef struct direct DIRENTRY ;
 
 
 
-  /* pathname of directory we store tape files in. */
+/* pathname of directory we store tape files in. */
 static char *tapeDirectory ;
 
-  /* the callback ID of the checkpoint timer callback. */
+/* the callback ID of the checkpoint timer callback. */
 static TimeoutId checkPtId ;
 
-  /* number of seconds between tape checkpoints. */
+/* number of seconds between tape checkpoints. */
 static u_int tapeCkPtPeriod ;
 
-  /* global list of tapes so we can checkpoint them periodically */
+/* global list of tapes so we can checkpoint them periodically */
 static Tape *activeTapes ;
 
-  /* Size of the activeTapes array */
+/* Size of the activeTapes array */
 static size_t activeTapeSize ;
 
-  /* index of last element in activeTapes that's being used. */
+/* index of last element in activeTapes that's being used. */
 static size_t activeTapeIdx ;
 
+/* default limit of the size of output tapes. */
+static long defaultSizeLimit ;
 
-
-  /* a structure for temporary storage of articles. */
+/* a structure for temporary storage of articles. */
 typedef struct q_e_s
 {
     Article article ;
@@ -113,48 +114,52 @@ typedef struct q_e_s
 } *QueueElem ;
 
 
-  /* The Tape class type. */
+/* The Tape class type. */
 struct tape_s
 {
-      /* the pathname of the file the administrator can drop in by hand. */
+    /* the pathname of the file the administrator can drop in by hand. */
     char *handFilename ;
 
-      /* the pathname of the file the Tape will read from */
+    /* the pathname of the file the Tape will read from */
     char *inputFilename ;
 
-      /* the pathname of the file the Tape will write to. */
+    /* the pathname of the file the Tape will write to. */
     char *outputFilename ;
 
-      /* the pathname of the file used in locking */
+    /* the pathname of the file used in locking */
     char *lockFilename ;
 
-      /* the pathname of the file to store the checkpoint information. */
-    char *checkpointFilename ;
-
-      /* the peer we're doing this for. */
+    /* the peer we're doing this for. */
     char *peerName ;
     
     FILE *inFp ;                /* input FILE */
     FILE *outFp ;               /* output FILE */
 
-      /* the tape holds a small output queue in memory to avoid thrashing. */
+    /* the tape holds a small output queue in memory to avoid thrashing. */
     QueueElem head ;            
     QueueElem tail ;
     u_int qLength ;             /* amount on the queue */
 
     bool outputHasStuff ;       /* true if there are articles to be had */
     bool outputFileExists ;     /* true if we've created the output file */
+
+    long outputSize ;           /* the current size of the output file. */
+
+    /* the number of bytes we try to keep the output under. We actually
+       wait for the outputSize to get 10% greater than this amount before
+       shrinking the file down again. A value of zero means no limit. */
+    long outputLimit ;
     
+    bool scribbled ;            /* have we scribbled a checkpoint value in
+                                   the file. */
     long tellpos ;              /* for input file checkpointing. */
     bool changed ;              /* true if tape was read since last
                                    checkpoint or start. */
 
-      /* true if articles that are output are NOT later input. */
+    /* true if articles that are output are NOT later input. */
     bool noRotate ;     
 };
 
-void addPointerFreedOnExit (char *pointerToFree) ;
-
 
 static void checkpointTape (Tape tape) ;
 static void removeTapeGlobally (Tape tape) ;
@@ -164,14 +169,13 @@ static void tapeCheckpointCallback (TimeoutId id, void *d) ;
 
 
 
-
-  /* Create a new Tape object. There are three potential files involved in
-     I/O. 'peerName' is what the admin may have dropped in by
-     hand. 'peerName.input' is the file that was being used as input the last
-     time things were run. 'peerName.output' is the file that was being used
-     as output. The file 'peerName' is appended to 'peerName.input' (or
-     renamed if 'peerName.input' doesn't exist). Then 'peerName.output' is
-     appeneded (or renamed) to 'peerName.input' */
+/* Create a new Tape object. There are three potential files involved in
+   I/O. 'peerName' is what the admin may have dropped in by
+   hand. 'peerName.input' is the file that was being used as input the last
+   time things were run. 'peerName.output' is the file that was being used
+   as output. The file 'peerName' is appended to 'peerName.input' (or
+   renamed if 'peerName.input' doesn't exist). Then 'peerName.output' is
+   appeneded (or renamed) to 'peerName.input' */
 
 Tape newTape (const char *peerName, bool dontRotate)
 {
@@ -190,9 +194,6 @@ Tape newTape (const char *peerName, bool dontRotate)
   if (endsIn (peerName,LOCK_TAIL))
     die ("Sorry, can't have a peer name ending in \"%s\"",LOCK_TAIL) ;
 
-  if (endsIn (peerName,CKPT_TAIL))
-    die ("Sorry, can't have a peer name ending in \"%s\"",CKPT_TAIL) ;
-
   nt->peerName = strdup (peerName) ;
   
   nt->handFilename = MALLOC (pLen + dLen + 2) ;
@@ -211,16 +212,13 @@ Tape newTape (const char *peerName, bool dontRotate)
   ASSERT (nt->outputFilename != NULL) ;
   sprintf (nt->outputFilename,"%s/%s%s",tapeDirectory,peerName,OUTPUT_TAIL) ;
 
-  nt->checkpointFilename = MALLOC (pLen + dLen + strlen(CKPT_TAIL) + 2) ;
-  ASSERT (nt->checkpointFilename != NULL) ;
-  sprintf (nt->checkpointFilename,"%s/%s%s",tapeDirectory,peerName,CKPT_TAIL) ;
-
   nt->inFp = NULL ;
   nt->outFp = NULL ;
   
   nt->head = NULL ;
   nt->tail = NULL ;
-  
+
+  nt->scribbled = false ;
   nt->tellpos = 0 ;
   nt->qLength = 0 ;
 
@@ -237,13 +235,14 @@ Tape newTape (const char *peerName, bool dontRotate)
       FREE (nt->lockFilename) ;
       FREE (nt->inputFilename) ;
       FREE (nt->outputFilename) ;
-      FREE (nt->checkpointFilename) ;
       FREE (nt) ;
 
       return NULL ;
     }
   
   nt->noRotate = false ;        /* for first time prepare */
+  nt->outputSize = 0 ;
+  nt->outputLimit = defaultSizeLimit ;
 
   prepareFiles (nt) ;
 
@@ -297,7 +296,6 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
   fprintf (fp,"%s    input-file : %s\n", indent, tape->inputFilename) ;
   fprintf (fp,"%s    output-file : %s\n",indent, tape->outputFilename) ;
   fprintf (fp,"%s    lock-file : %s\n",indent, tape->lockFilename) ;
-  fprintf (fp,"%s    checkpoint-file : %s\n",indent,tape->checkpointFilename);
   fprintf (fp,"%s    peerName : %s\n",indent,tape->peerName) ;
   fprintf (fp,"%s    input-FILE : %p\n",indent, tape->inFp) ;
   fprintf (fp,"%s    output-FILE : %p\n",indent, tape->outFp) ;
@@ -326,20 +324,18 @@ void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
   fprintf (fp,"%s    no-rotate : %s\n",indent, boolToString (tape->noRotate));
   
   fprintf (fp,"%s}\n",indent) ;
-  
-  
 }
 
 
 
 
-  /* delete the tape. Spools the in-memory articles to disk. */
+/* delete the tape. Spools the in-memory articles to disk. */
 void delTape (Tape tape)
 {
   if (tape == NULL)
     return ;
 
-    /* flush out articles not yet gone to disk. */
+  /* flush out articles not yet gone to disk. */
   if (tape->head != NULL)
     {
       tapeOpen (tape) ;         /* will flush articles to disk */
@@ -358,7 +354,6 @@ void delTape (Tape tape)
   freeCharP (tape->inputFilename) ;
   freeCharP (tape->outputFilename) ;
   freeCharP (tape->lockFilename) ;
-  freeCharP (tape->checkpointFilename) ;
   freeCharP (tape->peerName) ;
              
   removeTapeGlobally (tape) ;
@@ -367,10 +362,10 @@ void delTape (Tape tape)
 }
 
 
-  /* Request the tape to hang onto the article information. If the output
-     side of the tape has been opened explicitly, then the article infor is
-     dumped straight to disk. Otherwise it's stored on a small queue. When
-     the queue gets too big it will be flushed to disk. */
+/* Request the tape to hang onto the article information. If the output
+   side of the tape has been opened explicitly, then the article infor is
+   dumped straight to disk. Otherwise it's stored on a small queue. When
+   the queue gets too big it will be flushed to disk. */
 void tapeTakeArticle (Tape tape, Article article)
 {
   QueueElem elem ;
@@ -383,11 +378,23 @@ void tapeTakeArticle (Tape tape, Article article)
     {
       tape->outputFileExists = true ;
       
-      fprintf (tape->outFp,"%s %s\n",
-               artFileName (article), artMsgId (article)) ;
+      fprintf (tape->outFp,"%s %s\n",artFileName (article),
+               artMsgId (article)) ;
 
+      /* I'd rather know where I am each time, and I don't trust all
+         fprintf's to give me character counts. */
+      tape->outputSize = ftell (tape->outFp) ;
+      
       fflush (tape->outFp) ;
       delArticle (article) ;
+
+      if (tape->outputLimit > 0 &&
+          tape->outputSize >= (tape->outputLimit * LIMIT_FUDGE))
+        {
+          (void) shrinkfile (tape->outFp,
+                             tape->outputLimit,tape->outputFilename,"a+") ;
+          tape->outputSize = ftell (tape->outFp) ;
+        }
     }
   else                          /* stick on queue */
     {
@@ -414,8 +421,8 @@ void tapeTakeArticle (Tape tape, Article article)
 }
 
 
-  /* Pick an article off a tape and return it. NULL is returned if there
-     are no more articles. */
+/* Pick an article off a tape and return it. NULL is returned if there
+   are no more articles. */
 Article getArticle (Tape tape)
 {
   char line [2048] ;            /* ick. 1024 for filename + 1024 for msgid */
@@ -446,9 +453,9 @@ Article getArticle (Tape tape)
           dprintf (1,"No more articles on tape %s\n",tape->inputFilename) ;
 
           (void) unlink (tape->inputFilename) ;
-          (void) unlink (tape->checkpointFilename) ;
 
           tape->inFp = NULL ;
+          tape->scribbled = false ;
           prepareFiles (tape) ; /* rotate files to try next. */
         }
       else
@@ -456,7 +463,7 @@ Article getArticle (Tape tape)
           msgid = filename = NULL ;
           
           for (p = line ; *p && isspace (*p) ; p++) /* eat whitespace */
-              /* nada */ ;
+            /* nada */ ;
 
           if (*p != '\0')
             {
@@ -468,7 +475,7 @@ Article getArticle (Tape tape)
                   *q = '\0' ;
       
                   for (q++ ; *q && isspace (*q) ; q++) /* eat more white */
-                      /* nada */ ;
+                    /* nada */ ;
 
                   if (*q != '\0')
                     {
@@ -491,26 +498,26 @@ Article getArticle (Tape tape)
           if (filename != NULL && msgid != NULL)
             art = newArticle (filename, msgid) ;
 
-            /* art may be NULL here if the file is no longer valid. */
+          /* art may be NULL here if the file is no longer valid. */
         }
     }
 
   
-    /* now we either have an article or there is no more on disk */
+  /* now we either have an article or there is no more on disk */
   
   if (tape->inFp != NULL && ((c = fgetc (tape->inFp)) != EOF))
     ungetc (c,tape->inFp) ;
   else if (tape->inFp != NULL)
     {
-        /* last article read was the end of the tape. */
+      /* last article read was the end of the tape. */
       if (fclose (tape->inFp) != 0)
         syslog (LOG_ERR,FCLOSE_FAILED,tape->inputFilename) ;
       
       tape->inFp = NULL ;
+      tape->scribbled = false ;
 
-        /* toss out the old input file and prepare the new one */
+      /* toss out the old input file and prepare the new one */
       (void) unlink (tape->inputFilename) ;
-      (void) unlink (tape->checkpointFilename) ;
 
       prepareFiles (tape) ;
     }
@@ -526,7 +533,7 @@ Article getArticle (Tape tape)
 }
 
 
-  /* (re)open a tape for output. */
+/* (re)open a tape for output. */
 bool tapeOpen (Tape tape)
 {
   QueueElem elem ;
@@ -535,20 +542,23 @@ bool tapeOpen (Tape tape)
 
   if (tape->outFp == NULL)
     {
-      if ((tape->outFp = fopen (tape->outputFilename,"a")) == NULL)
+      if ((tape->outFp = fopen (tape->outputFilename,"a+")) == NULL)
         {
-          syslog (LOG_ERR,TAPE_OPEN_FAILED, "a", tape->outputFilename) ;
+          syslog (LOG_ERR,TAPE_OPEN_FAILED, "a+", tape->outputFilename) ;
           return false ;
         }
+
+      tape->outputSize = ftell (tape->outFp) ;
     }
 
-    /* flush out queue to disk. */
+  /* flush out queue to disk. */
   elem = tape->head ;
   while (elem != NULL)
     {
       tape->head = tape->head->next ;
-      fprintf (tape->outFp,"%s %s\n",artFileName (elem->article),
+      fprintf (tape->outFp,"%s %s\n", artFileName (elem->article),
                artMsgId (elem->article)) ;
+
       tape->outputFileExists = true ;
       delArticle (elem->article) ;
       
@@ -556,16 +566,25 @@ bool tapeOpen (Tape tape)
       elem = tape->head ;
     }
 
+  tape->outputSize = ftell (tape->outFp) ;
   tape->tail = NULL ;
   tape->qLength = 0;
       
   fflush (tape->outFp) ;
 
+  if (tape->outputLimit > 0 &&
+      tape->outputSize >= (tape->outputLimit * LIMIT_FUDGE))
+    {
+      (void) shrinkfile (tape->outFp,
+                         tape->outputLimit,tape->outputFilename,"a+") ;
+      tape->outputSize = ftell (tape->outFp) ;
+    }
+
   return true ;
 }
 
 
-  /* close the output side of the tape. */
+/* close the output side of the tape. */
 void tapeClose (Tape tape)
 {
   struct stat st;
@@ -582,12 +601,13 @@ void tapeClose (Tape tape)
     }
   
   tape->outFp = NULL ;
+  tape->outputSize = 0 ;
 }
 
 
 
-  /* return true if there are articles on the tape somewhere avialable for
-     processing. In the NOROTATE case we only look at the input side. */
+/* return true if there are articles on the tape somewhere avialable for
+   processing. In the NOROTATE case we only look at the input side. */
 bool tapeHasArticles (Tape tape)
 {
   bool rval = false ;
@@ -604,20 +624,20 @@ bool tapeHasArticles (Tape tape)
 }
 
 
-  /****************************************************/
-  /**                  CLASS FUNCTIONS               **/
-  /****************************************************/
+/****************************************************/
+/**                  CLASS FUNCTIONS               **/
+/****************************************************/
 
 
-  /* set the number of seconds between checkpoints of the input side of a
-     tape. */
+/* set the number of seconds between checkpoints of the input side of a
+   tape. */
 void tapeSetCheckpointPeriod (u_int ckPtPeriod)
 {
   tapeCkPtPeriod = ckPtPeriod ;
 }
 
 
-  /* Cause all the Tapes to checkpoint themselves. */
+/* Cause all the Tapes to checkpoint themselves. */
 void checkPointTapes (void)
 {
   u_int i ;
@@ -627,8 +647,8 @@ void checkPointTapes (void)
 }
 
 
-  /* Set the pathname of the directory for storing tapes in. can only set
-     it once, and before any tapes are created.  */
+/* Set the pathname of the directory for storing tapes in. can only set
+   it once, and before any tapes are created.  */
 void setTapeDirectory (const char *newDir)
 {
   ASSERT (tapeDirectory == NULL) ;
@@ -639,7 +659,7 @@ void setTapeDirectory (const char *newDir)
 }
 
 
-  /* Get the pathname of the directory tapes are stored in. */
+/* Get the pathname of the directory tapes are stored in. */
 const char *getTapeDirectory ()
 {
   if (tapeDirectory == NULL)
@@ -652,19 +672,22 @@ const char *getTapeDirectory ()
 }
 
 
+void setOutputSizeLimit (long val)
+{
+  defaultSizeLimit = val ;
+}
 
 
 
 
 
 
-
-  /**********************************************************************/
-  /*                          PRIVATE FUNCTIONS                         */
-  /**********************************************************************/
+/**********************************************************************/
+/*                          PRIVATE FUNCTIONS                         */
+/**********************************************************************/
 
 
-  /* Add a new tape to the class-level list of active tapes.  */
+/* Add a new tape to the class-level list of active tapes.  */
 static void addTapeGlobally (Tape tape)
 {
   ASSERT (tape != NULL) ;
@@ -688,7 +711,7 @@ static void addTapeGlobally (Tape tape)
 }
 
 
-  /* Remove a tape for the class-level list of active tapes. */
+/* Remove a tape for the class-level list of active tapes. */
 static void removeTapeGlobally (Tape tape)
 {
   u_int i ;
@@ -717,38 +740,61 @@ static void removeTapeGlobally (Tape tape)
 }
 
 
-  /* Have a tape checkpoint itself so that next process can pick up where
-     this one left off. */
+/* Have a tape checkpoint itself so that next process can pick up where
+   this one left off. */
 static void checkpointTape (Tape tape)
 {
-  FILE *tout ;
-  
   if (tape->inFp == NULL)       /* no input file being read. */
     return ;
 
   if (!tape->changed)   /* haven't read since last checkpoint */
     {
-      dprintf (1,"Not checkpointing unchanged tape: %s\n",
-               tape->checkpointFilename) ;
+      dprintf (1,"Not checkpointing unchanged tape: %s\n", tape->peerName) ;
       return ;
     }
   
   if ((tape->tellpos = ftell (tape->inFp)) < 0)
     {
       syslog (LOG_ERR,FTELL_FAILED,tape->inputFilename) ;
-      (void) unlink (tape->checkpointFilename) ;
 
       return ;
     }
-  
-  if ((tout = fopen (tape->checkpointFilename,"w")) == NULL)
-    syslog (LOG_ERR,CHECKPOINT_OPEN,tape->checkpointFilename) ;
-  else
-    {
-      fprintf (tout,"%ld\n",tape->tellpos) ;
 
-      if (fclose (tout) != 0)
-        syslog (LOG_ERR, FCLOSE_FAILED, tape->checkpointFilename) ;
+  /* strlen of "18446744073709551616\n" (2^64) */
+#define BITS64 21
+
+  /* make sure we're not right at the beginning of the file so we can write. */
+  if (tape->tellpos > BITS64)
+    {
+      rewind (tape->inFp) ;
+
+      /* scribble blanks over the first lines characters */
+      if (!tape->scribbled) 
+        {
+          int currloc = 0 ;
+          int c ;
+
+          while ((c = fgetc (tape->inFp)) != '\n' || currloc <= BITS64)
+            if (c == EOF)
+              return ;
+            else
+              currloc++ ;
+
+          rewind (tape->inFp) ;
+          
+          while (currloc-- > 0)
+            fputc (' ',tape->inFp) ;
+
+          rewind (tape->inFp) ;
+
+          fflush (tape->inFp) ;
+          tape->scribbled = true ;
+        }
+      
+      fprintf (tape->inFp,"%ld",tape->tellpos) ;
+
+      if (fseek (tape->inFp,tape->tellpos,SEEK_SET) != 0)
+        syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos) ;
     }
   
   tape->changed = false ;
@@ -760,8 +806,9 @@ static void checkpointTape (Tape tape)
  * three possible input files: PEER.input PEER and
  * PEER.output. PEER.input and PEER.output are private to
  * innfeed. PEER is where a sysadmin can drop a file that (s)he
- * wants to inject into the process.  There is a also a checkpojint
- * file that relates to the .input file.
+ * wants to inject into the process. If the first line of the input file
+ * contains only an integer (possibly surrounded by spaces), then this is
+ * taken to be the position to seek to
  *
  * prepareFiles will them in a manner much like the following shell
  * commands:
@@ -782,35 +829,21 @@ static void prepareFiles (Tape tape)
   bool inpExists ;
   bool outExists ;
   bool newExists ;
-  bool ckptExists ;
 
-    /* flush any in memory articles to disk */
+  /* flush any in memory articles to disk */
   if (tape->head != NULL)
     {
       tapeOpen (tape) ;
       tapeClose (tape) ;
     }
 
+  tape->tellpos = 0 ;
+  
   inpExists = fileExistsP (tape->inputFilename) ;
   outExists = fileExistsP (tape->outputFilename) ;
   newExists = fileExistsP (tape->handFilename) ;
-  ckptExists = fileExistsP (tape->checkpointFilename) ;
 
-
-    /* we blast the checkpoint file if the input file doesn't exist, or if
-       the input file is newer */
-  if (ckptExists)
-    {
-      if (!inpExists || isOlder (tape->checkpointFilename,tape->inputFilename))
-        {
-          if (unlink (tape->checkpointFilename) != 0)
-            syslog (LOG_ERR, UNLINK_FAILED, tape->checkpointFilename) ;
-          
-          ckptExists = false ;
-        }
-    }
-  
-    /* move the hand-dropped file to the input file if needed. */
+  /* move the hand-dropped file to the input file if needed. */
   if (newExists && !inpExists)
     {
       if (rename (tape->handFilename,tape->inputFilename) != 0)
@@ -819,8 +852,8 @@ static void prepareFiles (Tape tape)
         inpExists = true ;
     }
   
-    /* now move the output file to the input file, if needed and only if in
-       not in NOROTATE mode. */
+  /* now move the output file to the input file, if needed and only if in
+     not in NOROTATE mode. */
   if (outExists && !inpExists && !tape->noRotate)
     {
       if (rename (tape->outputFilename,tape->inputFilename) != 0)
@@ -832,81 +865,83 @@ static void prepareFiles (Tape tape)
       tape->outputFileExists = false ;
     }
 
-    /* get the checkpoint value if possible */
-  tape->tellpos = 0 ;
-  if (ckptExists)
-    {
-      long tellVal ;
-      int rval ;
-      FILE *ckptFile ;
-
-      if ((ckptFile = fopen (tape->checkpointFilename,"r")) == NULL)
-        syslog (LOG_ERR,FOPEN_FAILURE,tape->checkpointFilename) ;
-      else
-        {
-          char cbuff [50] ;
-          size_t len = 0 ;
-
-          rval = 1 ;
-
-          if (fgets (cbuff,sizeof (cbuff) - 1,ckptFile) == NULL)
-            rval = 0 ;
-          else
-            {
-              len = strlen (cbuff) ;
-              if (cbuff [len - 1] == '\n')
-                cbuff [--len] = '\0' ;
-            }
-
-          if (rval && (strspn (cbuff,"0123456789") != len))
-            rval = 0 ;
-          
-          if (rval)
-            rval = sscanf (cbuff,"%ld",&tellVal) ;
-          
-          if (fclose (ckptFile) != 0)
-            syslog (LOG_ERR,FCLOSE_FAILED,tape->checkpointFilename) ;
-
-          if (rval == 1)
-            tape->tellpos = tellVal ;
-          else
-            {
-              syslog (LOG_ERR,BAD_CHECKPOINT,tape->checkpointFilename) ;
-              
-              if (unlink (tape->checkpointFilename) != 0)
-                syslog (LOG_ERR, UNLINK_FAILED, tape->checkpointFilename) ;
-            }
-        }
-    }
-  
-
-    /* now open up the input file and seek to the proper position. */
+  /* now open up the input file and seek to the proper position. */
   if (inpExists)
     {
+      int c ;
       long flength ;
       
-      if ((tape->inFp = fopen (tape->inputFilename,"r")) == NULL)
+      if ((tape->inFp = fopen (tape->inputFilename,"r+")) == NULL)
         syslog (LOG_ERR,FOPEN_FAILURE,tape->inputFilename) ;
+      else
+        {
+          char buffer [64] ;
 
-        /* make sure we're not trying to seek past the end. */
+          if (fgets (buffer,sizeof (buffer) - 1, tape->inFp) == NULL)
+            {
+              syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
+              fclose (tape->inFp) ;
+              tape->inFp = NULL ;
+            }
+          else
+            {
+              u_int len = strlen (buffer) ;
+              long newPos = 0 ;
+
+              if (len > 0 && buffer [len - 1] == '\n')
+                buffer [--len] = '\0' ;
+
+              if (len > 0 && strspn (buffer,"0123456789 \n") == len)
+                {
+                  if (sscanf (buffer,"%ld",&newPos) == 1)
+                    {
+                      tape->scribbled = true ;
+                      tape->tellpos = newPos ;
+                    }
+                }
+            }
+        }
+      
       if ((flength = fileLength (fileno (tape->inFp))) < tape->tellpos)
         {
           syslog (LOG_ERR,FILE_SHORT,tape->inputFilename,
                   flength, tape->tellpos);
           tape->tellpos = 0 ;
-
-          if (unlink (tape->checkpointFilename) != 0)
-            syslog (LOG_ERR, UNLINK_FAILED, tape->checkpointFilename) ;
         }
-
-      if (fseek (tape->inFp,tape->tellpos,SEEK_SET) != 0)
+      else if (tape->tellpos == 0)
+        rewind (tape->inFp) ;
+      else if (fseek (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
         syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos);
+      else if ((c = fgetc (tape->inFp)) != '\n')
+        {
+          while (c != EOF && c != '\n')
+            c = fgetc (tape->inFp) ;
+
+          if (c == EOF)
+            {
+              fclose (tape->inFp) ;
+              unlink (tape->inputFilename) ;
+              tape->inFp = NULL ;
+              tape->scribbled = false ;
+              prepareFiles (tape) ;
+            }
+          else
+            {
+              long oldPos = tape->tellpos ;
+
+              tape->changed = true ;
+              checkpointTape (tape) ;
+              
+              syslog (LOG_ERR,CKPT_BNDRY,tape->inputFilename,
+                      tape->tellpos, oldPos) ;
+            }
+        }
     }
 }
 
 
 
-  /* The timer callback function that will checkpoint all the active tapes. */
+/* The timer callback function that will checkpoint all the active tapes. */
 static void tapeCheckpointCallback (TimeoutId id, void *d)
 {
   (void) d ;                    /* keep lint happy */
@@ -920,3 +955,6 @@ static void tapeCheckpointCallback (TimeoutId id, void *d)
 
   checkPtId = prepareSleep (tapeCheckpointCallback,tapeCkPtPeriod,NULL) ;
 }
+
+
+
